@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -312,7 +312,7 @@ class CUserGroup extends CZBXAPI {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can delete user groups.'));
 		}
 
-		$usrgrpids = zbx_toArray($data['usrgrpids']);
+		$usrgrpids = array_keys(array_flip(zbx_toArray($data['usrgrpids'])));
 		$userids = (isset($data['userids']) && !is_null($data['userids'])) ? zbx_toArray($data['userids']) : null;
 		$rights = (isset($data['rights']) && !is_null($data['rights'])) ? zbx_toArray($data['rights']) : null;
 
@@ -359,8 +359,8 @@ class CUserGroup extends CZBXAPI {
 			$linkedRights = array();
 			$sql = 'SELECT groupid,id'.
 				' FROM rights'.
-				' WHERE '.dbConditionInt('groupid', $usrgrpids);
-			' AND '.dbConditionInt('id', zbx_objectValues($rights, 'id'));
+				' WHERE '.dbConditionInt('groupid', $usrgrpids).
+					' AND '.dbConditionInt('id', zbx_objectValues($rights, 'id'));
 			$linkedRightsDb = DBselect($sql);
 			while ($link = DBfetch($linkedRightsDb)) {
 				if (!isset($linkedRights[$link['groupid']])) $linkedRights[$link['groupid']] = array();
@@ -370,7 +370,7 @@ class CUserGroup extends CZBXAPI {
 			$rightInsert = array();
 			foreach ($usrgrpids as $usrgrpid) {
 				foreach ($rights as $right) {
-					if (!isset($linkedUsers[$usrgrpid][$right['id']])) {
+					if (!isset($linkedRights[$usrgrpid][$right['id']])) {
 						$rightInsert[] = array(
 							'groupid' => $usrgrpid,
 							'permission' => $right['permission'],
@@ -452,9 +452,12 @@ class CUserGroup extends CZBXAPI {
 
 			// check whether user tries to add himself to a disabled user group
 			$usrgrps = $this->get(array(
+				'output' => array('usrgrpid', 'gui_access', 'users_status'),
 				'usrgrpids' => $usrgrpids,
-				'output' => API_OUTPUT_EXTEND,
+				'selectUsers' => array('userid'),
+				'preservekeys' => true
 			));
+
 			if (uint_in_array(self::$userData['userid'], $userids)) {
 				foreach ($usrgrps as $usrgrp) {
 					if (($usrgrp['gui_access'] == GROUP_GUI_ACCESS_DISABLED)
@@ -464,34 +467,32 @@ class CUserGroup extends CZBXAPI {
 				}
 			}
 
-			// get already linked users
-			$linkedUsers = array();
-			$sql = 'SELECT usrgrpid,userid'.
-				' FROM users_groups'.
-				' WHERE '.dbConditionInt('usrgrpid', $usrgrpids);
-			$linkedUsersDb = DBselect($sql);
-			while ($link = DBfetch($linkedUsersDb)) {
-				if (!isset($linkedUsers[$link['usrgrpid']])) {
-					$linkedUsers[$link['usrgrpid']] = array();
-				}
-				$linkedUsers[$link['usrgrpid']][$link['userid']] = 1;
-			}
-
-			// get user-userGroup links to insert and get user ids to unlink
 			$userUsergroupLinksToInsert = array();
-			$userIdsToUnlink = array();
-			foreach ($usrgrpids as $usrgrpid) {
+			$amount_of_usrgrps_to_unlink = array();
+
+			foreach ($usrgrps as $usrgrpid => $usrgrp) {
+				$usrgrp_userids = array();
+
+				foreach ($usrgrp['users'] as $user) {
+					$usrgrp_userids[$user['userid']] = true;
+				}
+
 				foreach ($userids as $userid) {
-					if (!isset($linkedUsers[$usrgrpid][$userid])) {
+					if (!array_key_exists($userid, $usrgrp_userids)) {
 						$userUsergroupLinksToInsert[] = array(
 							'usrgrpid' => $usrgrpid,
-							'userid' => $userid,
+							'userid' => $userid
 						);
 					}
-					unset($linkedUsers[$usrgrpid][$userid]);
+					else {
+						unset($usrgrp_userids[$userid]);
+					}
 				}
-				if (isset($linkedUsers[$usrgrpid]) && !empty($linkedUsers[$usrgrpid])) {
-					$userIdsToUnlink = array_merge($userIdsToUnlink, array_keys($linkedUsers[$usrgrpid]));
+
+				foreach ($usrgrp_userids as $userid => $value) {
+					$amount_of_usrgrps_to_unlink[$userid] = array_key_exists($userid, $amount_of_usrgrps_to_unlink)
+						? $amount_of_usrgrps_to_unlink[$userid] + 1
+						: 1;
 				}
 			}
 
@@ -500,10 +501,27 @@ class CUserGroup extends CZBXAPI {
 				DB::insert('users_groups', $userUsergroupLinksToInsert);
 			}
 
-			// unlink users from user groups
-			if (!empty($userIdsToUnlink)) {
+			if ($amount_of_usrgrps_to_unlink) {
+				// Unlink users from user groups.
+
+				$userids_to_unlink = array_keys($amount_of_usrgrps_to_unlink);
+
+				$db_users = API::User()->get(array(
+					'output' => array('userid', 'alias'),
+					'userids' => $userids_to_unlink,
+					'selectUsrgrps' => array('usrgrps')
+				));
+
+				foreach ($db_users as $user) {
+					if (count($user['usrgrps']) <= $amount_of_usrgrps_to_unlink[$user['userid']]) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('User "%1$s" cannot be without user group.', $user['alias'])
+						);
+					}
+				}
+
 				DB::delete('users_groups', array(
-					'userid' => $userIdsToUnlink,
+					'userid' => $userids_to_unlink,
 					'usrgrpid' => $usrgrpids,
 				));
 			}
@@ -678,14 +696,16 @@ class CUserGroup extends CZBXAPI {
 		));
 
 		foreach ($dbUsers as $dbUser) {
-			if (count($dbUser['usrgrps']) == 1) {
-				$dbGroup = reset($dbUser['usrgrps']);
+			$db_usrgrpids = array();
 
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-					'User group "%1$s" is the only group that user "%2$s" belongs to.',
-					$dbUserGroups[$dbGroup['usrgrpid']]['name'],
-					$dbUser['alias']
-				));
+			foreach ($dbUser['usrgrps'] as $usrgrp) {
+				$db_usrgrpids[] = $usrgrp['usrgrpid'];
+			}
+
+			if (!array_diff($db_usrgrpids, $userGroupIds)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('User "%1$s" cannot be without user group.', $dbUser['alias'])
+				);
 			}
 		}
 	}
